import { shallowMount } from '@vue/test-utils';
import Component from '@/components/JSONEditor.vue';

const factory = (data = {}, props = {}, methods = {}) =>
  shallowMount(Component, {
    data: () => ({ ...data }),
    propsData: { ...props },
    methods,
  });

const mockProp = {
  content: {
    test: [
      {
        test1: 'hello',
        test2: 'world',
      },
    ],
  },
};

describe('JSON Editor', () => {
  it('given a content object it should flatten it to vm.data', () => {
    const wrapper = factory({}, mockProp);

    expect(wrapper.vm.data).toEqual({
      'test.0.test1': 'hello',
      'test.0.test2': 'world',
    });
  });

  it('unflatten data to json object on save', () => {
    const downloadJSON = jest.fn();
    const expected = mockProp.content;
    const wrapper = factory({}, {
      content: mockProp.content,
      filename: 'test-filename',
    }, { downloadJSON });
    const DATE_TO_USE = new Date('Tue Aug 28 2018 13:15');

    global.Date = jest.fn(() => DATE_TO_USE);

    expect(wrapper.vm.save()).toEqual(expected);
    expect(downloadJSON).toHaveBeenCalledTimes(1);
    expect(downloadJSON).toHaveBeenCalledWith(expected, 'test-filename.20188281315');
  });

  it('unflatten data to json array on save', () => {
    const mockContentArray = {
      content: [
        { test1: 'hello' },
        { test2: 'world' },
      ],
    };
    const downloadJSON = jest.fn();
    const expected = mockContentArray.content;
    const wrapper = factory({}, mockContentArray, { downloadJSON });

    expect(wrapper.vm.save()).toEqual(expected);
  });

  it('will be able to update a value in the data object', () => {
    const wrapper = factory({}, mockProp);

    expect(wrapper.vm.updateValue('test.0.test2', 'moto')).toEqual({
      'test.0.test1': 'hello',
      'test.0.test2': 'moto',
    });
  });

  it('add a new object to an array', () => {
    const wrapper = factory({}, {
      content: {
        'influencer.products.0.handle': '@ryxn_dh',
        'influencer.products.0.quote': 'Every day camo and POD',
        'influencer.products.1.handle': '@cedric_castex',
        'influencer.products.1.quote': 'All the vibes',
      },
    });

    const expected = {
      'influencer.products.0.handle': '@ryxn_dh',
      'influencer.products.0.quote': 'Every day camo and POD',
      'influencer.products.1.handle': '@cedric_castex',
      'influencer.products.1.quote': 'All the vibes',
      'influencer.products.2.handle': '',
      'influencer.products.2.quote': '',
    };

    expect(wrapper.vm.addObjectToArray('influencer.products')).toEqual(expected);
    expect(wrapper.vm.data).toEqual(expected);
  });

  it('add a new object to an array at the top level', () => {
    const wrapper = factory({}, {
      content: {
        '0.handle': 'test 1',
        '0.quote': 'test 1',
      },
    });

    const expected = {
      '0.handle': 'test 1',
      '0.quote': 'test 1',
      '1.handle': '',
      '1.quote': '',
    };

    expect(wrapper.vm.addObjectToArray('')).toEqual(expected);
    expect(wrapper.vm.data).toEqual(expected);
  });

  it('can return an array from an object path to array', () => {
    const data = {
      'influencer.deep.nest.products.0.handle': 'array 0',
      'influencer.deep.nest.products.0.quote': 'array 0',
      'influencer.deep.nest.products.1.handle': 'array 1',
      'influencer.deep.nest.products.1.quote': 'array 1',
    };

    const wrapper = factory({}, {
      content: data,
    });

    expect(wrapper.vm.keyValuePairToArray({
      path: 'influencer.deep.nest.products',
      name: 'products',
      key: 'influencer.deep.nest',
    }, data))
      .toEqual({
        actual: [
          { handle: 'array 0', quote: 'array 0' },
          { handle: 'array 1', quote: 'array 1' },
        ],
        flat: {
          'influencer.deep.nest.products.0.handle': 'array 0',
          'influencer.deep.nest.products.0.quote': 'array 0',
          'influencer.deep.nest.products.1.handle': 'array 1',
          'influencer.deep.nest.products.1.quote': 'array 1',
        },
      });
  });

  it('create a new flat array object based on exist object', () => {
    const data = {
      '0.deep.nest.array.products.0.handle': 'array 0',
      '0.deep.nest.array.products.0.quote': 'array 0',
      '0.deep.nest.array.products.1.handle': 'array 1',
      '0.deep.nest.array.products.1.quote': 'array 1',
    };

    const wrapper = factory({}, {
      content: data,
    });

    expect(wrapper.vm.createFlatArrayObject('0.deep.nest.array.products')).toEqual({
      '0.deep.nest.array.products.2.handle': '',
      '0.deep.nest.array.products.2.quote': '',
    });
  });

  it('can return array from an array at the top level of object', () => {
    const data = {
      '0.handle': 'array 0',
      '0.quote': 'array 0',
      '1.handle': 'array 1',
      '1.quote': 'array 1',
    };

    const wrapper = factory({}, {
      content: data,
    });

    expect(wrapper.vm.keyValuePairToArray({
      path: '', name: '', key: '',
    }, data).actual)
      .toEqual([
        { handle: 'array 0', quote: 'array 0' },
        { handle: 'array 1', quote: 'array 1' },
      ]);
  });

  it('delete an object from an array', () => {
    const wrapper = factory({}, {
      content: {
        'influencer.deep.nest.products.0.handle': 'array 0',
        'influencer.deep.nest.products.0.quote': 'array 0',
        'influencer.deep.nest.products.1.handle': 'array 1',
        'influencer.deep.nest.products.1.quote': 'array 1',
        'influencer.deep.nest.products.2.handle': 'array 2',
        'influencer.deep.nest.products.2.quote': 'array 2',
        'influencer.deep.nest.products.3.handle': 'array 3',
        'influencer.deep.nest.products.3.quote': 'array 3',
      },
    });

    const expected = {
      'influencer.deep.nest.products.0.handle': 'array 0',
      'influencer.deep.nest.products.0.quote': 'array 0',
      'influencer.deep.nest.products.1.handle': 'array 2',
      'influencer.deep.nest.products.1.quote': 'array 2',
      'influencer.deep.nest.products.2.handle': 'array 3',
      'influencer.deep.nest.products.2.quote': 'array 3',
    };

    expect(wrapper.vm.deleteArrayObject('influencer.deep.nest.products.1')).toEqual(expected);
    expect(wrapper.vm.data).toEqual(expected);
  });

  it('delete an array from the top level', () => {
    const wrapper = factory({}, {
      content: {
        '0.handle': 'array 0',
        '0.quote': 'array 0',
        '1.handle': 'array 1',
        '1.quote': 'array 1',
        '2.handle': 'array 2',
        '2.quote': 'array 2',
      },
    });

    const expected = {
      '0.handle': 'array 0',
      '0.quote': 'array 0',
      '1.handle': 'array 2',
      '1.quote': 'array 2',
    };

    expect(wrapper.vm.deleteArrayObject('1')).toEqual(expected);
    expect(wrapper.vm.data).toEqual(expected);
  });

  it('parse an object index', () => {
    const wrapper = factory({}, mockProp);

    expect(wrapper.vm.parseArrayIndex('influencer.deep.nest.products.1')).toEqual({
      path: 'influencer.deep.nest.products',
      key: 'influencer.deep.nest',
      name: 'products',
      position: 1,
    });

    expect(wrapper.vm.parseArrayIndex('influencer.deep.nest.products')).toEqual({
      path: 'influencer.deep.nest.products',
      key: 'influencer.deep.nest',
      name: 'products',
    });

    expect(wrapper.vm.parseArrayIndex('1')).toEqual({
      key: '',
      name: '',
      path: '',
      position: 1,
    });
  });

  it('should have a preview button that saves to localStorage', () => {
    const wrapper = factory({}, mockProp);
    const previewBtn = wrapper.find('.btn-preview');

    expect(previewBtn.exists()).toBe(true);
  });

  it('should emit an event when saving if remote content is true', () => {
    const wrapper = factory({}, { remoteContent: true, content: {} });
    const saveBtn = wrapper.find('.btn-download');

    saveBtn.trigger('click');
    expect(wrapper.emitted().save[0]).toEqual([{ content: {} }]);
  });

  it('should only emit scheudule in payload object when Schedule Release is true', () => {
    const wrapper = factory({ schedule: true }, { remoteContent: true, content: {} });
    const saveBtn = wrapper.find('.btn-download');

    saveBtn.trigger('click');
    expect(wrapper.emitted().save[0]).toEqual([{ content: {}, schedule: {} }]);
  });
});
